#include <iostream>
#include <fstream>
#include <string>
using namespace std;

struct Person {
    string name;
    double paid;
    double owed;
};

class Node {
public:
    Person data;
    Node* next;
    Node(const Person& person) : data(person), next(nullptr) {}
};

class LinkedList {
private:
    Node* head;
    Node* tail;
public:
    LinkedList() : head(nullptr) {}
    ~LinkedList();

    Node* get_head() const { return head;}
    void append_node(const Person& person);
    void update_owed_amounts(double averageBill);
};

void read_file(ifstream& f, LinkedList& list, double& sum_paid, int& people);
void clear_bill(LinkedList& list);

int main() {
    ifstream f;
    double sum_paid = 0;
    int people = 0;
    LinkedList personList;
    read_file(f, personList, sum_paid, people);

    double average_paid = sum_paid / people;
    personList.update_owed_amounts(average_paid);

    Node* current = personList.get_head();
    while (current != nullptr) {
        if (current->data.owed == 0) {
            cout << current->data.name
                 << ", you don't need to do anything." << endl;
        }
        current = current->next;
    }

    clear_bill(personList);
    cout.precision(2);
    cout << "In the end, you should all have spent around $" << average_paid << endl;
    return 0;
}

void LinkedList::append_node(const Person& person) {
    if (head == nullptr) {
        head = new Node(person);
        tail = head;
    } else {
        tail->next = new Node(person);
        tail = tail->next;
    }
}

void LinkedList::update_owed_amounts(double averageBill) {
    Node* current = head;
    while (current != nullptr) {
        current->data.owed = current->data.paid - averageBill;
        current = current->next;
    }
}

LinkedList::~LinkedList() {
    while (head != nullptr) {
        Node* temp = head;
        head = head->next;
        delete temp;
    }
}

void read_file(ifstream& f, LinkedList& list, double& sum_paid, int& people) {
    string file_name;
    cout << "Please enter file name:\n";
    cin >> file_name;
    f.open(file_name);
    while (!f.is_open()) {
        cout << "Please enter file name:\n";
        cin >> file_name;
        f.open(file_name);
    }

    Person temp;
    while (f >> temp.paid) {
        sum_paid += temp.paid;
        ++people;
        getline(f >> ws, temp.name);
        list.append_node(temp);
    }
    f.close();
}

void clear_bill(LinkedList& list) {
    Node* less_than_avg = list.get_head();
    Node* more_than_avg = list.get_head();

    cout.setf(ios::fixed);
    cout.setf(ios::showpoint);
    cout.precision(6);

    while (less_than_avg != nullptr && more_than_avg != nullptr) {
        while (less_than_avg != nullptr && less_than_avg->data.owed >= 0) {
            less_than_avg = less_than_avg->next;
        }
        while (more_than_avg != nullptr && more_than_avg->data.owed <= 0) {
            more_than_avg = more_than_avg->next;
        }

        if (less_than_avg == nullptr || more_than_avg == nullptr) {
            break; // No more transactions can be made
        }

        if (-1 * less_than_avg->data.owed <= more_than_avg->data.owed) {
            cout << less_than_avg->data.name
                 << ", you give " << more_than_avg->data.name
                 << " $" << -1 * less_than_avg->data.owed << endl;

            more_than_avg->data.owed += less_than_avg->data.owed;
            less_than_avg->data.owed = 0;
        } else {
            cout << less_than_avg->data.name
                 << ", you give " << more_than_avg->data.name
                 << " $" << more_than_avg->data.owed << endl;

            less_than_avg->data.owed += more_than_avg->data.owed;
            more_than_avg->data.owed = 0;
        }
    }
}